local assert = require("luassert")
local ts = require("ltreesitter")
local util = require("spec.util")

describe("ltreesitter.load", function()
	it("should return nil and an error message on failure", function()
		local ok, err = ts.load("./parser_that_doesnt_exist", "language_that_doesnt_exist")
		assert(ok == nil, "Expected load to return nil")
		assert(type(err) == "string", "Expected an error message")
	end)
	it("should return a ltreesitter.TSParser", function()
		local p = ts.load(os.getenv("HOME") .. "/dev/parsers/teal.so", "teal")
		util.assert_userdata_type(p, "ltreesitter.TSParser")
	end)
end)

describe("ltreesitter.require", function()
	it("should error on failure", function()
		assert.has.errors(function() ts.require("non_existent_thing") end)
	end)
	it("should return a ltreesitter.TSParser", function()
		local p = ts.require("teal")
		util.assert_userdata_type(p, "ltreesitter.TSParser")
	end)
end)
